
#include "INA219I2C.h"

// Battery control
const float vBatTarget = 13.8;  // バッテリー充電電圧の目標値
const float vBatG1 = 0.1;       // 目標値+/-0.1Vに収まるようにPWMを調整
const float vBatG2 = 0.6;       // 大きく外れた場合は調整幅を大きくする
const float cBatTarget = 1;     // バッテリー充電電流の目標値
const float cBatG1 = 0.1;      // 目標値+/-0.1Aに収まるようにPWMを調整
const float cBatG2 = 0.5;       // 大きく外れた場合は調整幅を大きくする

// Pin number
const uint8_t pinSW2 = 0;
const uint8_t pinSW3 = 4;
const uint8_t pinSW4 = 16;
const uint8_t pinSW5 = 5;
const uint8_t pinSDCS = 17;
const uint8_t pinSDA = 21;
const uint8_t pinSCL = 22;
const uint8_t pinPWM = 32;
const uint8_t pinLE = 13;
const uint8_t pinLoad = 33;
const uint8_t pinLED1 = 25;
const uint8_t pinLED2 = 26;
const uint8_t pinLED3 = 27;
const uint8_t pinLED4 = 14;
const uint8_t pinADC = 36;

INA219I2C ina219(pinSDA, pinSCL);     // INA219制御クラス
uint16_t pwm = 0;                     // 現在のPWMデューティー


void setup() {
  Serial.begin(115200);
  delay(10);
  Serial.print("\n\n-----------------------\n");
  Serial.print("Program Start\n");

  
  // Pin
  digitalWrite(pinLE, LOW);
  digitalWrite(pinLoad, HIGH);  // HIGH = OFF
  digitalWrite(pinLED1, HIGH);  // HIGH = OFF
  digitalWrite(pinLED2, HIGH);  // HIGH = OFF
  digitalWrite(pinLED3, HIGH);  // HIGH = OFF
  digitalWrite(pinLED4, HIGH);  // HIGH = OFF
  pinMode(pinLE, OUTPUT);
  pinMode(pinLoad, OUTPUT);
  pinMode(pinLED1, OUTPUT);
  pinMode(pinLED2, OUTPUT);
  pinMode(pinLED3, OUTPUT);
  pinMode(pinLED4, OUTPUT);
  pinMode(pinSW2, PULLUP);
  pinMode(pinSW3, PULLUP);
  pinMode(pinSW4, PULLUP);
  pinMode(pinSW5, PULLUP);
  ledcSetup(0, 500, 8);      // PWM 500Hz, 8bit(0-256)
  ledcAttachPin(pinPWM, 0);   // PWMピン割り当て

  // INA219
  ina219.init();  // 初期化
}


void loop() {
  pwmControl();  
  delay(100);
}


// PWM制御
void pwmControl(){
  float vBat = ina219.vbus();     // バッテリー電圧取得
  float cBat = ina219.current();  // バッテリー電流取得

  // 目標の電圧、電流に応じてPWMデューティーを変更
  if (vBat > vBatTarget + vBatG2 || cBat > cBatTarget + cBatG2) {
    if (pwm > 0 && pwm <= 50) {
      pwm -= 1;
    } else {
      pwm -= 5;
    }
  } else if (vBat > vBatTarget + vBatG1 || cBat > cBatTarget + cBatG1) {
    if (pwm >= 1) {
      pwm -= 1;
    }
  } else if (vBat < vBatTarget - vBatG2 && cBat < cBatTarget - cBatG2) {
    if (pwm >= 246) {
      pwm = 256;
    } else if (pwm <= 50){
      pwm += 1;
    } else {
      pwm += 5;
    }
  } else if (vBat < vBatTarget - vBatG1 && cBat < cBatTarget - cBatG1) {
    if (pwm <= 255) {
      pwm += 1;
    }
  }

  ledcWrite(0, pwm);

  // シリアルモニタに表示
  Serial.print("PWM:");
  Serial.print(pwm);
  Serial.print(" V:");
  Serial.print(vBat);
  Serial.print(" C:");
  Serial.println(cBat);
}
